#!/bin/bash

# function for echo'ing bold text
function echobold () {
    echo -e "\e[1m$1\e[0m"
}

# check user not root
if [ $USER == "root" ]; then
    echobold "ERROR: This script must be NOT be run as root user!"
    exit
fi

# check currently loaded kernel
ver=$(uname -r)
echo ""
echo -n "You are currently booted into kernel "
echobold "$ver"
echo -n "Continue installing LinuxCNC on this kernel? Y/n: "
read resp
if [[ $resp != "Y" && $resp != "y" && $resp != "" ]]; then
    echo "NOT installing LinuxCNC!"
    exit
fi

# check if online
echo -n "Checking if online..."
if [ "`ping -q -c 3 google.com 2>/dev/null`" ]; then
    echobold "OK"
else
    echobold "FAIL"
    echo "ERROR: This script requires a live internet connection."
    exit
fi
sleep 3

# update apt sources
echo "Running 'aptitude update'..."
echo "Please enter your root password..."
su root -c "aptitude update"
if [ $? != 0 ]; then
    echobold "ERROR: see above output"
    exit
fi

# build LINUXCNC
echo -n "Building LinuxCNC as user "
echobold "'$USER'"
sleep 3
echo -n "Checking if gcc4.6 is installed..."
ver=$(gcc --version 2>/dev/null)
if [[ ! $ver =~ " 4.6." ]]; then
    echobold "FAIL"
    echobold "Installing and linking gcc4.6..."
    echobold "Please enter your root password..."
    cmd="aptitude -y install build-essential cpp-4.6 gcc-4.6 g++-4.6 libstdc++6-4.6-dev"
    cmd="$cmd && rm /usr/bin/cpp /usr/bin/gcc /usr/bin/gcov /usr/bin/g++"
    cmd="$cmd && ln -s /usr/bin/cpp-4.6 /usr/bin/cpp"
    cmd="$cmd && ln -s /usr/bin/gcc-4.6 /usr/bin/gcc"
    cmd="$cmd && ln -s /usr/bin/g++-4.6 /usr/bin/g++"
    cmd="$cmd && ln -s /usr/bin/gcov-4.6 /usr/bin/gcov"
    su root -c "$cmd"
    if [ $? != 0 ]; then
        echobold "ERROR: see above output"
        exit
    fi
    # re-check gcc install
    echo -n "Checking if gcc4.6 was successfully installed..."
    ver=$(gcc --version 2>/dev/null)
    if [[ ! $ver =~ " 4.6." ]]; then
        echobold "FAIL"
        echobold "You will need to manually install and link gcc4.6."
        echobold "Exiting..."
        exit
    else
        echobold "OK"
    fi
else
    echobold "OK"
fi
sleep 3

# check if git is installed
echo -n "Checking if git is installed..."
resp=$(git --version &>/dev/null)
if [ $? != 0 ]; then
    is_git=false
    echobold "FAIL"
    echobold "Installing git. Please enter your root password..."
    su root -c "aptitude -y install git-core gitk git-gui"
    if [ $? != 0 ]; then
        echobold "ERROR: see above output"
        exit
    fi
    # check install
    resp=$(git --version &>/dev/null)
    if [ $? == 0 ]; then
        is_git=true
    fi
else
    is_git=true
    echobold "OK"
fi
sleep 3

if $is_git; then
    # get linuxcnc sources
    target_path="/home/$USER/linuxcnc-dev"
    if [ ! -d $target_path ]; then
        echobold "Fetching LinuxCNC source..."
        git clone git://git.linuxcnc.org/git/linuxcnc.git $target_path
    fi

    echo -n "Checking if $target_path exists..."
    if [ -d $target_path ]; then
        echobold "YES"
        sleep 3
        echo "Checking for LinuxCNC updates..."
        sleep 3
        cd $target_path
        git pull
        resp=$(debian/configure -r)
        substring="successfully configured for"
        if [[ ! $resp =~ "$substring" ]]; then
            echo $resp
            echobold "ERROR: the 'debian/configure -a' command failed. See above output for details."
            exit
        fi

        echo "Checking dependencies..."
        sleep 3
        deps=$(dpkg-checkbuilddeps 2>&1>/dev/null)
        deps=${deps##*: } # trim to text following last ": "
        cmd=""
        for dep in ${deps//:/ } ; do # filter package names
            if [[ $dep =~  [a-z]+ ]]; then
                cmd="$cmd $dep" # append package name
            fi
        done
        
        cmd="libtk-img python-gnome2 python-glade2 mesa-utils gedit $cmd"
        echobold "Installing dependencies..."
        echo "$cmd"
        echo "Please enter your root password..."
        cmd = "aptitude -y install $cmd"
        su root -c "$cmd"
        if [ $? != 0 ]; then
            echobold "ERROR: see above output"
            echo "TIP: try running 'aptitude install -f' to fix broken packages"
            exit
        fi
        echobold "All dependencies are satisfied"

        echo -n "Cleaning..."
        sleep 3
        cd "$target_path/src"
        make clean &>/dev/null
        cd $target_path
        git clean -xdf &>/dev/null
        echobold "OK"
        sleep 3
        echo -n "Configuring..."
        sleep 3
        cd "$target_path/src"
        ./autogen.sh
        resp=$(./configure)

        # check if configure was successful
        substring="configure completed successfully"
        if [[ ! $resp =~ "$substring" ]]; then
            echobold "FAIL"
            echo "ERROR: the './configure' command failed."
            sleep 3
            echo "Printing output..."
            sleep 5
            echo $resp
            echo ""
            echobold "This must be resolved before re-running this script."
            exit
        else
            echobold "OK"
        fi

        # compile
        echo "Running 'make'..."
        sleep 3
        make
        echo ""
        echo "Setting user id. Please enter your root password..."
        su root -c "make setuid"
        if [ $? != 0 ]; then
            echobold "ERROR: see above output"
            exit
        fi
        echobold "DONE"
        echo ""
        echobold "You can now run LinuxCNC with the following commands:"
        echo "  cd $target_path"
        echo "  . scripts/rip-environment"
        echo "  linuxcnc"
        echo ""
        echobold "NOTE: If LinuxCNC starts but exits with an error like:"
        echo "  'RTAPI: Locked memory limit is 64KiB, recommended at least 20480KiB.'"
        echobold "See here for a solution:"
        echo "  http://wiki.linuxcnc.org/cgi-bin/wiki.pl?LockedMemory"
        exit
    else
        echobold "NO"
        echobold "ERROR: $target_path was not created."
        echo "Try re-running this script"
    fi
else
    echobold "ERROR: git not installed!"
    echobold "Try re-running this script or install git manually."
fi

exit
